function disp_img = calcDisparity(...
    left_img, right_img, patch_size, min_disp, max_disp)
% TODO: compute disparity image between left and right image 
% conventions: 
% - return a disparity image with the same size like the input images
% - set disparity to 0 if the patch comparison is undefined
% - patches are of size patch_size x patch_size and centered at the pixel so
% patch_size should be odd
% - keep disparities within [min_disp, max_disp] interval

disp_img = zeros(size(left_img));
rad = floor(patch_size / 2);

width = size(left_img, 2);
height = size(left_img, 1);

for v = rad + 1 : height - rad
    for u = rad + 1 : width - rad
        
        x_patch = left_img(v - rad : v + rad, u - rad : u + rad);
        x = x_patch(:)';
        
        y = [];
        
        for d = min_disp : max_disp
            if u - d < rad + 1
                break;
            end
            
            y_patch = right_img(v - rad : v + rad, u - d - rad : u - d + rad);
            y = [y; y_patch(:)'];
        end
            
        if size(y, 1) > 0
            % SAD
            [D, I] = pdist2(y, x, 'cityblock', 'Smallest', 3);  
            
            % SSD
            %[D, I] = pdist2(y, x, 'squaredeuclidean', 'Smallest', 3);   
            
             disp_img(v, u) = I(1, 1) + min_disp - 1;
%                      
            if (size(y, 1) > 2 && D(3, 1) / D(1, 1) < 1.5 && D(2, 1) / D(1, 1) < 1.5)
                disp_img(v, u) = 0;
            else
                disp_img(v, u) = I(1, 1) + min_disp - 1;
            end
        else
            disp_img(v, u) = 0;
        end
    end

end

