clear all;
close all;

left_img = imresize(imread('../../data/kitti/left/000000.png'), 0.5);
right_img = imresize(imread('../../data/kitti/right/000000.png'), 0.5);
K = load('../../data/kitti/K.txt');
K(1:2, :) = K(1:2, :) / 2;

% according to kitti dataset
baseline = 0.54;

patch_size = 11;
min_disp = 5;
max_disp = 50;
xlims = [7 20];
ylims = [-6 10];
zlims = [-5 5];

% compute disparity on the image pair
tic;
disp_img = calcDisparity(left_img, right_img, patch_size, min_disp, max_disp);
toc
figure(1);
imagesc(disp_img);
axis equal;
axis off;
% visualize 3D point cloud
[points, intensities] = disparity2PointCloud(...
    disp_img, K, baseline, left_img);

% transform pc from camera frame to world frame
points = [0 -1 0; 0 0 -1; 1 0 0]^-1 * points(:, 1:10:end);

figure(3);
scatter3(points(1, :), points(2, :), points(3, :), ...
    20 * ones(1, length(points)), ...
    repmat(single(intensities(1:10:end))'/255, [1 3]), 'filled');
axis equal;
axis([0 30 ylims zlims]);
axis vis3d;
grid off;
xlabel('X');
ylabel('Y');
zlabel('Z');

return

% create disparity movie for whole sequence
figure(2);
for i = 0:99
    l = imresize(imread(sprintf('../data/kitti/left/%06d.png',i)), 0.5);
    r = imresize(imread(sprintf('../data/kitti/right/%06d.png',i)), 0.5);
    disp_img_i = calcDisparity(l, r, patch_size, min_disp, max_disp);
    im = imagesc(disp_img_i);
    axis equal;
    axis off;
    pause(0.01);
    F(i) = im2frame(uint8(abs(disp_img_i)),gray);
end
movie(F)

