% read in the measurement file
ys = load( '../data/map_measurements.txt' );
%size(xs)

r = 0.2; %better than 0.05

p_free = 0.3;
p_occ = 0.7;
p_prior = 0.5;

logpocc_threshold = log( 0.9 / 0.1 );

min_img_y = -0.2;
max_img_y = 0.2;
img_res = 0.01;

zs = [min_img_y:img_res:max_img_y];
xs = ys .* repmat(zs, 20, 1);

% max grid positions x/y: [-1.6,1.6]
max_grid_pos = 1.6;
grid_resolution = 0.01; %0.1
grid_pos = [-max_grid_pos:grid_resolution:max_grid_pos];

logodds_grid = zeros( size(grid_pos,2), size(grid_pos,2) );

figure(1);
plot( xs, ys, 'r.' );
axis([-max_grid_pos,max_grid_pos,-max_grid_pos,max_grid_pos]);


% project the grid cells into the image and integrate distance to surface
for i=1:size(logodds_grid,1)
    for j=1:size(logodds_grid,2)
        
        gridx = grid_pos(i);
        gridy = grid_pos(j);
        
        gridz = gridx / gridy;
        
        grid_img_y = int32(round( (gridz-min_img_y) / img_res ))+1;

        if grid_img_y >= 1 && grid_img_y <= size( zs, 2 )
            
            % go through images and lookup distance measurement
            for k=1:size(ys,1)
                dist_y = dot([xs(k,grid_img_y);ys(k,grid_img_y)],[0;1]);
                dist_m = dot([gridx;gridy],[0;1]);
                
                if dist_m < 0
                    continue;
                end
                
                pmgy = p_prior;
                if dist_m < dist_y - r/2
                    pmgy = p_free;
                elseif dist_m >= dist_y - r/2 && dist_m <= dist_y + r/2
                    pmgy = p_occ;
                end
                
                logodds_grid(i,j) = logodds_grid(i,j) + log(pmgy/(1-pmgy));
                
            end
            
        end
        
    end
    
end


figure(2);
imagesc( logodds_grid )



% cast ray from camera center through measurement to find first occupied
% cell
raycast_resolution = 0.5*grid_resolution;
ray_max_step =  max_grid_pos / raycast_resolution;

logodds_grid2 = 0*logodds_grid;

% for k=1:1
%     for j=10:10
for k=1:size(ys,1)
   for j=1:size(ys,2)
        
        ray_dir = [xs(k,j);ys(k,j)] / norm([xs(k,j);ys(k,j)]);
        
        for s=0:ray_max_step
            ray_grid_pos = s*raycast_resolution * ray_dir;
            ray_grid_idx = int32(round((ray_grid_pos + max_grid_pos) / grid_resolution)) + 1;
            
            if logodds_grid(ray_grid_idx(1), ray_grid_idx(2)) > logpocc_threshold
                
                logodds_grid2(ray_grid_idx(1), ray_grid_idx(2)) = 1;
                break;
                            
            end

        end
    end
end

% visualize the found occupied cells
figure(3);
imagesc( logodds_grid2 )

% p(m|y) could be defined, but p(y|m) is hard, e.g., if we know some cell m
% is free, it's difficult to define where the measurement is.


