% read in the measurement file
ys = load( '../data/map_measurements.txt' );
%size(xs)

eta_1 = 0.1;
eta_2 = 0.2;

epsilon = 0.001;

min_img_y = -0.2;
max_img_y = 0.2;
img_res = 0.01;

zs = [min_img_y:img_res:max_img_y];
xs = ys .* repmat(zs, 20, 1);

% max grid positions x/y: [-1.6,1.6]
max_grid_pos = 1.6;
grid_resolution = 0.05; %0.1
grid_pos = [-max_grid_pos:grid_resolution:max_grid_pos];
sdf_grid_d = zeros( size(grid_pos,2), size(grid_pos,2) );
sdf_grid_w = zeros( size(grid_pos,2), size(grid_pos,2) );
sdf_grid = zeros( size(grid_pos,2), size(grid_pos,2) );

figure(1);
plot( xs, ys, 'r.' );
axis([-max_grid_pos,max_grid_pos,-max_grid_pos,max_grid_pos]);


% project the grid cells into the image and integrate distance to surface
for i=1:size(sdf_grid,1)
    for j=1:size(sdf_grid,2)
        
        gridx = grid_pos(i);
        gridy = grid_pos(j);
        
        gridz = gridx / gridy;
        
        % pixel centers are at the given values
        grid_img_y = int32(round( (gridz-min_img_y) / img_res )) + 1;

        if grid_img_y >= 1 && grid_img_y <= size( zs, 2 )
            
            
            % go through images and lookup distance measurement
            for k=1:size(ys,1)
                diff = [gridx;gridy] - [xs(k,grid_img_y);ys(k,grid_img_y)];
                dist = norm(diff);
                signed_dist = sign( dot([0;1], diff) ) * dist;
                
                weight = 0;
                if dist <= eta_1
                    weight = 1;
                elseif dist <= eta_2
                    weight = 1 - (dist-eta_1)/eta_2;
                end
                
                sdf_grid_d(i,j) = sdf_grid_d(i,j) + weight * signed_dist;
                sdf_grid_w(i,j) = sdf_grid_w(i,j) + weight;
                
            end
            
        end
        
    end
    
end

for i=1:size(sdf_grid,1)
    for j=1:size(sdf_grid,2)
        if sdf_grid_w(i,j) > epsilon
            sdf_grid(i,j) = sdf_grid_d(i,j)/sdf_grid_w(i,j);
        else
            sdf_grid(i,j) = 0;
        end
    end
end

figure(2);
imagesc( sdf_grid )
colorbar;


% cast ray from camera center through first measurement to find zero
% crossing
raycast_resolution = 0.5*grid_resolution;
ray_max_step =  max_grid_pos / raycast_resolution;

sdf_grid2 = sdf_grid;

xs_zc = 0*xs;
ys_zc = 0*ys;

% for k=1:1
%     for j=10:10
for k=1:size(ys,1)
   for j=1:size(ys,2)
        
        ray_dir = [xs(k,j);ys(k,j)] / norm([xs(k,j);ys(k,j)]);
        
        last_sdf_value = 0;
        last_grid_pos = [0,0];
        last_grid_idx = [-1,-1];
        
        for s=0:ray_max_step
            s_curr = s;
            ray_grid_pos = s*raycast_resolution * ray_dir;
            ray_grid_idx = int32(round((ray_grid_pos + max_grid_pos) / grid_resolution)) + 1;

            if sdf_grid_w(ray_grid_idx(1), ray_grid_idx(2)) > epsilon
                
                sdf_value = sdf_grid_d(ray_grid_idx(1), ray_grid_idx(2)) / sdf_grid_w(ray_grid_idx(1), ray_grid_idx(2));

                if last_sdf_value < 0 && sdf_value >= 0
                    
                    % search locally around the found transition as we might
                    % miss the zero crossing exactly on the ray in the found cells
                    % due to round
                    for ds=-2:1:+3

                        % found zero crossing, now find zero crossing with sub-cell
                        % accuracy
                        
                        % bilinearly interpolate sdf values at last and
                        % current ray position
                        
                        ray_grid_pos0 = (s+ds-1)*raycast_resolution * ray_dir;
                        ray_grid_pos1 = (s+ds)*raycast_resolution * ray_dir;
                        interp_sdf_value = interpolateGridValue( sdf_grid, (ray_grid_pos1 + max_grid_pos) / grid_resolution + 1 );
                        interp_last_sdf_value = interpolateGridValue( sdf_grid, (ray_grid_pos0 + max_grid_pos) / grid_resolution + 1 );

                        if interp_last_sdf_value < 0 && interp_sdf_value >= 0
                            sdf_grid2(ray_grid_idx(1), ray_grid_idx(2)) = 10;
                            delta_s = findZeroCrossing( interp_last_sdf_value, interp_sdf_value );
                            ray_grid_pos_deltas = (s+ds-1+delta_s)*raycast_resolution * ray_dir;
                            interp_sdf_value = interpolateGridValue( sdf_grid, (ray_grid_pos_deltas + max_grid_pos) / grid_resolution + 1 );

                            xs_zc(k,j) = ray_grid_pos_deltas(1);
                            ys_zc(k,j) = ray_grid_pos_deltas(2);
                            
                            break;
                        end
                    end

                else
                    sdf_grid2(ray_grid_idx(1), ray_grid_idx(2)) = max( [sdf_grid2(ray_grid_idx(1), ray_grid_idx(2)), 1] );
                end

                last_sdf_value = sdf_value;  
                last_grid_pos = ray_grid_pos;
                last_grid_idx = ray_grid_idx;

            end

        end
    end
end

% visualize the zero crossing search
figure(3);
imagesc( sdf_grid2 )

% visualize the found surface
figure(4);
plot( xs_zc, ys_zc, 'r.' );
axis([-max_grid_pos,max_grid_pos,-max_grid_pos,max_grid_pos]);



